// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "visualnovel/ui/VisualNovelTextbox.hpp"
#include "asset/assets/TextureAsset.hpp"
#include "asset/assets/TrueTypeAsset.hpp"

namespace Dawn {
  class PokerGameTextbox {
    public:
      static std::vector<Asset*> getAssets(AssetManager *man) {
        return std::vector<Asset*>{
          man->get<TrueTypeAsset>("truetype_ark"),
          man->get<TextureAsset>("texture_test")
        };
      }

      static VisualNovelTextbox * create(UICanvas *canvas) {
        auto assMan = &canvas->getGame()->assetManager;

        auto assetFont = assMan->get<TrueTypeAsset>("truetype_ark");
        auto assetTexture = assMan->get<TextureAsset>("texture_test");

        auto textbox = canvas->addElement<VisualNovelTextbox>();
        textbox->setBorder(&assetTexture->texture, glm::vec2(16, 16));
        textbox->setFont(&assetFont->font);
        textbox->setFontSize(40);
        textbox->setLabelPadding(glm::vec2(10, 8));
        textbox->setTransform(
          UI_COMPONENT_ALIGN_STRETCH, UI_COMPONENT_ALIGN_END,
          glm::vec4(0, 238, 0, 0),
          0.0f
        );
        return textbox;
      }
  };
}