// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "PokerPlayerDisplay.hpp"
#include "game/DawnGame.hpp"

using namespace Dawn;

PokerPlayerDisplay::PokerPlayerDisplay(UICanvas *canvas) :
  UIEmpty(canvas),
  labelName(canvas),
  labelChips(canvas)
{
  this->font = getGame()->assetManager.get<TrueTypeAsset>("truetype_ark");

  // Player Name
  this->addChild(&this->labelName); 
  this->labelName.setText("Player Name");
  this->labelName.setFont(&this->font->font);
  this->labelName.setFontSize(40);
  this->labelName.setTransform(
    UI_COMPONENT_ALIGN_START, UI_COMPONENT_ALIGN_START, 
    glm::vec4(0, 0, 0, 0),
    0.0f
  );

  // Chips label
  this->addChild(&this->labelChips);
  this->labelChips.setFont(&this->font->font);
  this->labelChips.setFontSize(40);
  this->labelChips.setTransform(
    UI_COMPONENT_ALIGN_START, UI_COMPONENT_ALIGN_START, 
    glm::vec4(0, this->labelName.getContentHeight(), 0, 0),
    0.0f
  );

  // Events
  getScene()->eventSceneUnpausedUpdate.addListener(this, &PokerPlayerDisplay::onSceneUpdate);
}

void PokerPlayerDisplay::setPlayer(PokerPlayer *player) {
  assertNull(this->player);
  this->player = player;

  player->eventChipsChanged.addListener(this, &PokerPlayerDisplay::onPlayerChipsChanged);

  this->labelName.setText("Name");
  this->animChips.clear();
  this->animChips.restart();
  this->animChipsValue = player->chips;

  this->updatePositions();
}

void PokerPlayerDisplay::onSceneUpdate() {
  this->animChips.tick(getGame()->timeManager.delta);

  std::stringstream stream;
  stream.precision(0);
  stream << "$";
  stream << this->animChipsValue;
  this->labelChips.setText(stream.str());
}

void PokerPlayerDisplay::onPlayerChipsChanged() {
  std::cout << "Chips" << player->chips << std::endl;

  this->animChips.clear();
  this->animChips.addKeyframe(&this->animChipsValue, 0.0f, this->animChipsValue);
  this->animChips.addKeyframe(&this->animChipsValue, 1.0f, this->player->chips);
  this->animChips.restart();
}

PokerPlayerDisplay::~PokerPlayerDisplay() {
  this->canvas->getScene()->eventSceneUnpausedUpdate.removeListener(this, &PokerPlayerDisplay::onSceneUpdate);
}